import Connection from "./Connection";

export default function App() {
  return (
    <div className="min-w-screen font-main min-h-screen bg-rock font-sen text-gray-400">
      <Connection />
    </div>
  );
}
