import "@fontsource/sen/400.css";
import "@fontsource/sen/700.css";
import "@fontsource/poppins/400.css";
import "@fontsource/poppins/700.css";
import "./index.css";

import App from "./components/App";
import ReactDOM from "react-dom/client";

ReactDOM.createRoot(document.getElementById("root") as HTMLElement).render(<App />);
